<?php
// phpcs:disable WordPress.PHP.DevelopmentFunctions
// phpcs:disable WordPress.Files.FileName.InvalidClassFileName
// phpcs:disable WordPress.WP.AlternativeFunctions.json_encode_json_encode
// phpcs:disable WordPress.Files.FileName.NotHyphenatedLowercase

use WeeConnectPay\Api\Requests\VerifyAuthenticationRequest;
use WeeConnectPay\Dependencies\GuzzleHttp\Exception\ClientException;
use WeeConnectPay\Exceptions\Codes\ExceptionCode;
use WeeConnectPay\Integrations\AdminPanel;
use WeeConnectPay\Integrations\PaymentFields;
use WeeConnectPay\WordPress\Plugin\includes\WeeConnectPayUtilities;
use WeeConnectPay\WordPress\Plugin\includes\WeeConnectPayAPI;
use WeeConnectPay\WordPress\Plugin\includes\WeeConnectPayHelper;
use WeeConnectPay\Exceptions\WeeConnectPayException;
use WeeConnectPay\Integrations\Authentication;
use WeeConnectPay\Integrations\IntegrationSettings;
use WeeConnectPay\Settings;
use WeeConnectPay\StandardizedResponse;

if ( ! class_exists( WC_Payment_Gateway::class ) ) {
	return null;
}

/**
 * Handles the WeeConnectPay/Clover/WooCommerce payment gateway
 */
class WC_Gateway_Weeconnectpay extends WC_Payment_Gateway {
	/**
	 * Instance of the WeeConnectPay API
	 * @var WeeConnectPayAPI $api
	 */
	private $api;
	/**
	 * @var string
	 */
	private $url_api;
	/**
	 * @var string
	 */
	private $integration_id;
	/**
	 * @var int
	 */
	private $authVerifyHttpCode;

	/**
	 * @return WeeConnectPayAPI
	 */
	public function getApi(): WeeConnectPayAPI {
		return $this->api;
	}

	/**
	 * @param WeeConnectPayAPI $api
	 *
	 * @return WC_Gateway_Weeconnectpay
	 */
	public function setApi( WeeConnectPayAPI $api ): WC_Gateway_Weeconnectpay {
		$this->api = $api;

		return $this;
	}

//	/**
//	 * JSON Web Token used to authorize communication with the WeeConnectPay API
//	 *
//	 * @var string
//	 */
//	private $weeconnectpay_access_token;

	private $is_debug = true;

	private $iframe_settings;

//	private $clover_api_key;

//	private $ownership_key;

	/**
	 * @var IntegrationSettings $integrationSettings
	 */
	private $integrationSettings;

	/**
	 * @return IntegrationSettings
	 */
	public function getIntegrationSettings(): IntegrationSettings {
		return $this->integrationSettings;
	}

	/**
	 * @param IntegrationSettings $integrationSettings
	 *
	 * @return WC_Gateway_Weeconnectpay
	 */
	public function setIntegrationSettings( IntegrationSettings $integrationSettings ): WC_Gateway_Weeconnectpay {
		$this->integrationSettings = $integrationSettings;

		return $this;
	}

	public function __construct() {

//		// In checkout page /?wc-ajax=update_order_review gets triggered and reloads - We want the gateway to load after ajax
//		if ( is_checkout() && ! is_wc_endpoint_url( 'order-pay' ) && ! is_ajax() ) {
//			return null;
//		}

//		// We don't do anything with the cart or order-received endpoints
//		if ( is_cart() || is_wc_endpoint_url( 'order-received' ) ) {
//			return null;
//		}

		$this->id                 = 'weeconnectpay';
		$this->method_title       = __( 'Clover Integration', 'weeconnectpay' );
		$this->method_description = __(
			'Simplify online payments by adding the Clover payment option to your shopping cart. Then you will see your payments in real time on your Clover web portal.'
			, 'weeconnectpay' );
		$this->has_fields         = true;
		$this->supports           = array(
			'refunds',
		);


		$this->init_form_fields();
		$this->init_settings();


		try {
			$integration_settings = new IntegrationSettings();
            $integration_settings->getIntegrationUuid();
		    // Save the integration settings as an attribute on this class
			$this->setIntegrationSettings( $integration_settings);
			// Save the API as an attribute on this class
			$this->setApi( new WeeConnectPayAPI() );
		} catch ( WeeConnectPayException $e ) {
			return StandardizedResponse::emitError( $e->toObject() );
		}

		$this->url_api = $this->api->url_api;
        $this->integration_id = $this->integrationSettings->getIntegrationUuid();


		$this->title       = $this->get_option( 'title' );
		$this->enabled     = $this->get_option( 'enabled' );
		$this->description = $this->get_option( 'description' );

		//Runs when we update the gateway options through woocommerce
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array(
			$this,
			'process_admin_options'
		) );

		// API Callback
		add_action( 'woocommerce_api_callback_' . strtolower( get_class( $this ) ), array(
			$this,
			'weeconnectpay_callback_handler'
		) );

//		add_action( 'woocommerce_checkout_order_processed', array(
//			$this,
//			'action_checkout_order_processed'
//		), 10, 3 ); // Only fires on checkout page, not order-pay

		add_action( 'woocommerce_after_checkout_validation', array( $this, 'maybe_add_wc_notice' ), 10, 2 );

		//add_action('woocommerce_checkout_after_order_review', array( $this, 'maybe_add_wc_notice' ) );
		//add_action('woocommerce_checkout_before_order_review', array( $this, 'maybe_add_wc_notice' ) );
		add_action( 'woocommerce_checkout_process', array( $this, 'maybe_add_wc_notice' ), 10, 2 );



	}

	/**
	 * Override the parent admin_options method with our app.
	 *
	 * @since 1.4.0
	 */
	public function admin_options() {
		if ( ! $this->integrationSettings->isAuthValid() ) {

            // Hide the save button from the payment gateway settings form
			global $hide_save_button;
			$hide_save_button = true;

			$redirect_url = $this->url_api . '/login/clover?intent=authorize-redirect&integration_id=' . $this->integration_id;
			$vue_data     = array(
				'redirectUrl' => $this->integrationSettings::redirectUrl(),
				'pluginUrl'   => WEECONNECTPAY_PLUGIN_URL,
				'redirectUrl' => $redirect_url,
			);

			$admin_panel = new AdminPanel();
			$admin_panel->init( $vue_data );
		} else {
			parent::admin_options();
		}
	}

	function maybe_add_wc_notice( $fields, WP_Error $errors = null ) {
		global $wp;

		// Reset prevent submit
		$_POST['weeconnectpay_prevent_submit'] = false;

		if ( wc_notice_count( 'error' ) !== 0 ) {
			return;
		}

		// Untouched form
		if ( isset( $_POST['weeconnectpay_prevent_submit_empty_cc_form'] ) ) {
			wc_add_notice( __( "Please enter your payment information.", 'weeconnectpay' ), 'error' );
			$_POST['weeconnectpay_prevent_submit'] = true;
		}

		// Card Number
		if ( isset( $_POST['weeconnectpay_prevent_submit_card_number_error_cc_form'] ) ) {
			wc_add_notice( __( "Please enter a valid credit card number.", 'weeconnectpay' ), 'error' );
			$_POST['weeconnectpay_prevent_submit'] = true;
		}
		if ( isset( $_POST['weeconnectpay_prevent_submit_empty_card_number_cc_form'] ) ) {
			wc_add_notice( __( "Please enter a valid credit card number.", 'weeconnectpay' ), 'error' );
			$_POST['weeconnectpay_prevent_submit'] = true;
		}

		// Expiry Date
		if ( isset( $_POST['weeconnectpay_prevent_submit_date_error_cc_form'] ) ) {
			wc_add_notice( __( "Please enter a valid credit card expiry date.", 'weeconnectpay' ), 'error' );
			$_POST['weeconnectpay_prevent_submit'] = true;
		}
		if ( isset( $_POST['weeconnectpay_prevent_submit_empty_date_cc_form'] ) ) {
			wc_add_notice( __( "Please enter a valid credit card expiry date.", 'weeconnectpay' ), 'error' );
			$_POST['weeconnectpay_prevent_submit'] = true;
		}

		// CVV
		if ( isset( $_POST['weeconnectpay_prevent_submit_cvv_error_cc_form'] ) ) {
			wc_add_notice( __( "Please enter a valid credit card CVV number.", 'weeconnectpay' ), 'error' );
			$_POST['weeconnectpay_prevent_submit'] = true;
		}
		if ( isset( $_POST['weeconnectpay_prevent_submit_empty_cvv_cc_form'] ) ) {
			wc_add_notice( __( "Please enter a valid credit card CVV number.", 'weeconnectpay' ), 'error' );
			$_POST['weeconnectpay_prevent_submit'] = true;
		}

		// Postal Code
		if ( isset( $_POST['weeconnectpay_prevent_submit_postal_code_error_cc_form'] ) ) {
			wc_add_notice( __( "Please enter a valid credit card postal code.", 'weeconnectpay' ), 'error' );
			$_POST['weeconnectpay_prevent_submit'] = true;
		}
		if ( isset( $_POST['weeconnectpay_prevent_submit_empty_postal_code_cc_form'] ) ) {
			wc_add_notice( __( "Please enter a valid credit card postal code.", 'weeconnectpay' ), 'error' );
			$_POST['weeconnectpay_prevent_submit'] = true;
		}
	}


	/**
	 * Generates the WeeConnectPay settings form fields for WooCommerce to use in the payment gateway settings
	 * @return void
	 * @since 1.0.0
	 * @updated 1.4.0
	 */
	public function init_form_fields() {

		$this->form_fields = array(
			'enabled'          => array(
				'title'       => __( 'Enable', 'weeconnectpay' ),
				'label'       => __( 'Enable payment gateway', 'weeconnectpay' ),
				'type'        => 'checkbox',
				'description' => '',
				'default'     => 'yes',
			),
			'authorize_button' => array(
				'title' => __( 'Authorize Plugin', 'weeconnectpay' ),
				'type'  => 'authorize_button',
			),
			'title'            => array(
				'title'       => __( 'Title', 'weeconnectpay' ),
				'type'        => 'text',
				'description' => __( 'This controls the title which the user sees during checkout.', 'weeconnectpay' ),
				'default'     => __( 'Payment by Credit Card - WeeConnectPay', 'weeconnectpay' ),
			),
		);
	}

	/**
	 * Screen button Field
	 */
	public function generate_authorize_button_html() {
		//$redirect_url = $this->url_api . '/login/clover?intent=authorize-redirect&integration_id=' . $this->integration_id;
        $redirect_url = IntegrationSettings::redirectUrl();

        ?>
        <tr valign="top">
            <td colspan="2" class="">
                <a href="<?php echo esc_url( $redirect_url); ?>" class="button"><?php
//					if ( $this->authVerifyHttpCode !== 200 ) {
//						_e( 'Authorize Plugin', 'weeconnectpay' );
//					} else {
						esc_html_e(  'Log in as another Clover merchant or employee', 'weeconnectpay' );
//					}
					?></a>
            </td>
        </tr>
		<?php
	}


	public function action_checkout_order_processed( int $order_id, array $posted_data, WC_Order $order ) {
		// Only happens at first checkout -- Not after payment declines the first time
		$this->prepare_order_and_authed_iframe( $order );
	}

	/* @TODO: Checks to prevent loading the gateway if it shouldn't
	 * @BODY: To check: Physical Location, Currency, etc
	 */
	public function payment_fields() {

		try {
			$script_data = array(
				'pakms'  => $this->integrationSettings->getPublicAccessKey(),
				'locale' => WeeConnectPayUtilities::getLocale(),
			);

			$payment_fields = new PaymentFields();
			$payment_fields->init( $script_data );
		} catch (Exception $exception) {
            error_log( 'Exception caught in payment_fields: '.$exception->getMessage());
		}
	}

	/**
	 * Tells WooCommerce whether the gateway should be available IE:( In checkout / order pay, etc ).
	 * @return bool
	 * @since 1.3.7
	 */
	public function is_available(): bool {

		// If WeeConnectPay is not enabled, return.
		if ( 'no' === $this->enabled ) {
			return false;
		}


		if ( ! ( new IntegrationSettings() )->arePaymentProcessingSettingsReady() ) {
			return false;
		}

		// SSL ( Even though plugin no longer loads without it )
		if ( ! is_ssl() ) {
			return false;
		}

		// TEMPORARY DISABLED UNTIL ORDER PAY LOGIC IS FIXED
		if ( is_wc_endpoint_url( 'order-pay' ) ) {
			//return false;
		}

		return true;
	}


	/**
	 * @inheritDoc
     * @updated 2.0.6
	 */
	public function process_payment( $order_id ) {
//		error_log( 'DEBUG: process_payment Post data: ' . json_encode( $_POST ) );
		$order = wc_get_order( $order_id );

        // Do we have a tokenized card?
		if (!empty($_POST['token'])) {
			$tokenizedCard = _sanitize_text_fields( $_POST['token']);
        } else {
			error_log( 'Missing tokenized card to process the payment with.' );
			return array(
				'result'   => 'fail',
				'redirect' => '',
			);
		}

		// Does the order already have a Clover order UUID?
		if ( ! $order->meta_exists( 'weeconnectpay_clover_order_uuid' ) ) {
			try {
				$order_response = $this->api->prepare_order_for_payment( $order );
			} catch ( WeeConnectPayException $exception ) {
				return $this->handleProcessPaymentException( $exception );
			}

            if ( isset( $order_response['id'] ) && isset( $order_response['uuid'] ) && isset( $order_response['amount'] ) ) {
				// Has the secure_uuid for the iframe to know what to pay
				$this->iframe_settings['order_payload'] = $order_response;

				// Metadata to check at order creation to prevent double orders
				$order->add_meta_data( 'weeconnectpay_clover_order_uuid', $order_response['uuid'] );
				$order->save_meta_data();
                $cloverOrderUuid = $order_response['uuid'];
			} else {
				error_log( "Something went wrong while creating the Clover order for WooCommerce order #$order_id" );
				return array(
					'result'   => 'fail',
					'redirect' => '',
				);
			}
		} else {
			$cloverOrderUuid = $order->get_meta( 'weeconnectpay_clover_order_uuid' );
		}

		error_log( 'CloverOrderUuid: '.json_encode( $cloverOrderUuid) );

		// Attempt to create a charge for the order
		try {
			$chargeResponse = ( new \WeeConnectPay\Api\Requests\CreateCloverOrderChargeRequest() )->POST( $cloverOrderUuid, $tokenizedCard);
		} catch ( ClientException $exception ) {

			error_log( "Failed to create a charge for WooCommerce order #$order_id. ".json_encode( $exception->getMessage()) );
			return array(
				'result'   => 'fail',
				'redirect' => '',
			);
		}

        // Handle the response
         error_log("chargeResponse: ".json_encode( $chargeResponse));

		return array(
			'result'   => 'success',
			'redirect' => $order->get_checkout_order_received_url(),
		);
	}


	/**
	 * iFrame payment callback handler. Called from the WeeConnectPay API server after a user attempts to pay for their order through the iFrame.
	 */
	public function weeconnectpay_callback_handler() {

		$raw_post = file_get_contents( 'php://input' );
		$decoded = json_decode( $raw_post );

		if ( isset( $decoded->woocommerce_order_id ) ) {
			$order = new WC_Order( $decoded->woocommerce_order_id );
			if ( $order ) {
				if ( isset( $decoded->clover_payment_status ) ) {
					if ( 'paid' === $decoded->clover_payment_status ) {
						$order->payment_complete();
						wc_add_notice( 'Payment accepted by Clover, the payment ID is ' . $decoded->clover_payment_id, 'success' );
						$order->add_order_note( 'Clover Payment accepted, the payment ID is ' . $decoded->clover_payment_id );
						// Deduct inventory API call
						$clover_inventory_in_order = $order->get_meta( 'weeconnectpay_clover_inventory_in_order' );

						if ( isset( $clover_inventory_in_order ) && '' !== $clover_inventory_in_order ) {
							$api                       = isset( $this->api ) ? $this->api : new WeeConnectPayAPI( $this->get_option_key() );
							$inventory_update_response = $api->update_inventory( 'clover', 'subtract', $clover_inventory_in_order );
						}
					} elseif ( 'failed' === $decoded->clover_payment_status ) {
						if ( isset( $decoded->error->charge ) ) {
							$order->add_order_note( 'Clover Payment failed. The payment ID is ' . $decoded->error->charge );
							if ( isset( $decoded->error->message ) ) {
								$order->add_order_note( 'Clover error message: ' . $decoded->error->message );
							}
							wc_add_notice( 'Payment refused by Clover, the payment ID is ' . $decoded->error->charge, 'error' );
						} else {
							$order->add_order_note( 'Payment failed - Unhandled context, see response payload: ' . json_encode( $decoded ) );
						}
						$order->update_status( 'failed' );
					} else {
						$order->add_order_note( 'Something wrong happened with the payment; Here is what we have received from Clover: ' . json_encode( $decoded, JSON_PRETTY_PRINT ) );
					}
				}
			} else {
			}
		}
		die();
	}

	public function process_refund( $order_id, $amount = null, $reason = null ) {

		/** @TODO: Add wpdb prefix on order creation call */
		// Needed for the DB prefix for multi site
		global $wpdb;
		$order = new WC_Order( $order_id );

		$tax_included            = $order->get_meta( 'weeconnectpay_tax_included' );
		$merged_qty              = $order->get_meta( 'weeconnectpay_merged_qty' );
		$shipping_as_line_item   = null;
		$shipping_line_item_name = null;
		$shipping_item           = array();

		// Get the WC_Order Object instance (from the order ID)
		if ( ! is_a( $order, 'WC_Order' ) ) {
			return new WP_Error( 'wc-order', __( 'Provided ID is not a WC Order', 'weeconnectpay' ) );
		}

		// Get the Order refunds (array of refunds)
		$order_refunds = $order->get_refunds();

		// Only get the last refund order created since we're only going to process the one we just created
		if ( ! isset( $order_refunds[0] ) ) {
			return new WP_Error( 'wc-order', __( 'No WC Order Refund found', 'weeconnectpay' ) );
		}
		$latest_refund = $order_refunds[0];

		// Make sure it's an order refund object
		if ( ! is_a( $latest_refund, 'WC_Order_Refund' ) ) {
			return new WP_Error( 'wc-order', __( 'Last created refund is not a WC Order Refund', 'weeconnectpay' ) );
		}

		// Make sure it's not already been refunded HERE ( Payment processor checks need to be done on our backend for other refund means )
		if ( 'refunded' === $latest_refund->get_status() ) {
			return new WP_Error( 'wc-order', __( 'Order has been already refunded', 'weeconnectpay' ) );
		}

		$line_items = array();
		// Potential polymorphic calls during iteration -- Better try/catch as Woocommerce "conveniently" marks the refund as complete if there's an unhandled exception.
		try {
			// Get all the line items to refund
			foreach ( $latest_refund->get_items() as $item_id => $item ) {

				// Original order line item
				$refunded_item_id    = $item->get_meta( '_refunded_item_id' );
				$refunded_item       = $order->get_item( $refunded_item_id );
				$refunded_item_array = array(
					'line_item_id'     => $refunded_item_id,
					'line_total'       => WeeConnectPayHelper::safe_amount_to_cents_int( $refunded_item->get_total() ),
					'line_total_tax'   => WeeConnectPayHelper::safe_amount_to_cents_int( $refunded_item->get_total_tax() ),
					'line_quantity'    => $refunded_item->get_quantity(),
					'line_description' => WeeConnectPayHelper::name_and_qty_as_clover_line_desc(
						$refunded_item->get_name(),
						$refunded_item->get_quantity()
					),
				);

				// Order Refund line item
				array_push(
					$line_items,
					array(
						// Quantity: zero or negative integer
						'refunded_quantity'    => $item->get_quantity(),
						// line total: zero or negative number
						'refunded_line_total'  => WeeConnectPayHelper::safe_amount_to_cents_int( $item->get_total() ),
						// line total tax: zero or negative number
						'refunded_total_tax'   => WeeConnectPayHelper::safe_amount_to_cents_int( $item->get_total_tax() ),
						'order_refund_item_id' => $item_id,
						'refunded_item'        => $refunded_item_array,
					)
				);

			}

			// Add shipping if it's part of the refund request
			if ( $latest_refund->get_shipping_total() + $latest_refund->get_shipping_tax() ) {
				$shipping_line_item_name = $order->get_meta( 'weeconnectpay_shipping_line_item_name' );
				$shipping_as_line_item   = $order->get_meta( 'weeconnectpay_shipping_as_clover_line_item' );
				// If there's an amount to refund related to shipping and the order to be refunded has the the shipping line item name AND shipping as line item metadata
				if ( $shipping_as_line_item && $shipping_line_item_name ) {

					$shipping_line_item_amount                 = WeeConnectPayHelper::safe_amount_to_cents_int( $latest_refund->get_shipping_total() ) + WeeConnectPayHelper::safe_amount_to_cents_int( $latest_refund->get_shipping_tax() );
					$shipping_item['refunded_shipping_amount'] = $shipping_line_item_amount;
					$shipping_item['refunded_shipping_name']   = $shipping_line_item_name;
				} else {
					return false;
				}
			}
		} catch ( Throwable $e ) {
			error_log( "DEBUG: Process refund first try/catch exception: " . $e->getMessage() );
			return false;
		}
		error_log( "DEBUG: Process refund AFTER first try/catch." );

		$formatted_number = WeeConnectPayHelper::safe_amount_to_cents_int( $amount );

		$refund_payload = array(
			'shipping_as_line_item' => '1' === $shipping_as_line_item,
			'tax_included'          => '1' === $tax_included,
			'merged_qty'            => '1' === $merged_qty,
			'woocommerce_order_id'  => $order_id,
			'wpdb_prefix'           => $wpdb->prefix,
			'amount'                => $formatted_number,
			'reason'                => $reason,
			'line_items'            => $line_items,
		);

		// Add shipping item to payload if it's being refunded as a line item
		if ( isset( $shipping_item['refunded_shipping_amount'] ) ) {
			$refund_payload['shipping_item'] = $shipping_item;
		}

		$refund_response = $this->api->refund_woocommerce_order( $refund_payload );
		error_log( "DEBUG: AFTER refund_woocommerce_order(refund_payload) Response: ".json_encode( $refund_response));
		if ($refund_response instanceof WP_Error){
			error_log( "DEBUG: AFTER refund_woocommerce_order and is instanceof WP_Error");

			return $refund_response;
        }
		if ( isset( $refund_response->id )
		     && isset( $refund_response->amount )
		     && isset( $refund_response->charge )
		     && isset( $refund_response->status )
		     && ( 'succeeded' === $refund_response->status )
		) {
			$formatted_refund_amount = number_format( (float) $refund_response->amount / 100, 2, '.', '' );
			if ( '' !== $reason ) {
				$reason = ' Reason: ' . $reason;
			}
			$order->add_order_note( "Refunded \$$formatted_refund_amount (Refund ID: $refund_response->id - Charge refunded: $refund_response->charge)$reason" );

			return true;
		} elseif ( isset( $refund_response->id )
		           && isset( $refund_response->amount_returned )
		           && isset( $refund_response->items )
		           && isset( $refund_response->status )
		           && ( 'returned' === $refund_response->status )
		) {

			$formatted_returned_amount = number_format( (float) $refund_response->amount_returned / 100, 2, '.', '' );
			if ( '' !== $reason ) {
				$reason = ' Reason: ' . $reason;
			}
			$order_note_item_string = "Refunded \$$formatted_returned_amount (Refund ID: $refund_response->id)$reason";
			foreach ( $refund_response->items as $item_returned ) {
				if ( isset( $item_returned->parent )
				     && isset( $item_returned->description )
				     && isset( $item_returned->amount )
				) {
					$clover_item_id                   = $item_returned->parent;
					$clover_item_returned_description = $item_returned->description ?? null;
					$formatted_return_amount          = number_format( (float) $item_returned->amount / 100, 2, '.', '' );

					$order_note_item_string .= "\nReturned clover item ID $clover_item_id(\$$formatted_return_amount), $clover_item_returned_description";
				} else {

				}
			}
			$order->add_order_note( $order_note_item_string );

			return true;
		} else {
			error_log( "very end of process_refund before false" );
			return new WP_Error( 'wc-order', __( 'Order has been already refunded', 'weeconnectpay' ) );
		}
	}

	/**
     * @deprecated since 1.5
	 * @param WC_Order $order
	 */
	private function prepare_order_and_authed_iframe( WC_Order $order ): void {
		// Do something right before we try to process the payment
		$order_response = $this->api->prepare_order_for_payment( $order );
		if ( isset( $order_response['id'] ) && isset( $order_response['uuid'] ) && isset( $order_response['amount'] ) ) {
			// Has the secure_uuid for the iframe to know what to pay
			$this->iframe_settings['order_payload'] = $order_response;

			// Metadata to check at order creation to prevent double orders
			$order->add_meta_data( 'weeconnectpay_clover_order_uuid', $order_response['uuid'] );
			$order->save_meta_data();
		}

	}

	/**
     * Used in process_payment. Handles a WeeConnectPayException to properly display a notice and return the failure array.
	 * @param WeeConnectPayException $exception
	 *
	 * @return string[]
     * @since 2.0.6
	 */
	protected function handleProcessPaymentException( WeeConnectPayException $exception ): array {
		if ( $exception->getCode() === ExceptionCode::MISSING_SHIPPING_STATE ) {
			wc_add_notice( esc_html( $exception->getMessage() ), 'error' );
		} else {
			error_log( 'An unhandled exception happened while preparing the order with the payment processor.' );
		}

		return array(
            'result'   => 'fail',
            'redirect' => '',
        );
	}

	/**
	 * @return bool
	 */
	private function is_authenticated_iframe(): bool {
		return isset( $this->iframe_settings, $this->iframe_settings['order_payload']['secure_uuid'] );
	}

	/**
	 * @return array|void
	 */
	protected function verifyAuthentication() {
		$wp_env = WeeConnectPayUtilities::get_wp_env();

		switch ( $wp_env ) {
			case 'local':
			case 'development':
				// Do dev stuff
				$url_api = 'https://weeconnect-api.test';
				break;
			case 'staging':
				// Do staging stuff
				$url_api = 'https://apidev.weeconnectpay.com';
				break;
			case 'production':
			default:
				// Do production stuff
				$url_api = 'https://api.weeconnectpay.com';
		}

		try {

			$integration_id = Authentication::fetchIntegrationId();

			$integrationSettings = new IntegrationSettings();
			if ( $integrationSettings->accessTokenExists() ) {
				$authVerifyHttpCode = Authentication::verify( $integration_id );
			} else {
				$authVerifyHttpCode = 401;
			}

		} catch ( WeeConnectPayException $exception ) {
			die( json_encode( StandardizedResponse::emitError( $exception->toObject() ) ) );
		}

		return array( $url_api, $integration_id, $authVerifyHttpCode );
	}

}


