<?php

namespace WeeConnectPay\Api\Requests;

use WeeConnectPay\Dependencies\GuzzleHttp\Exception\GuzzleException;
use WeeConnectPay\Dependencies\Psr\Http\Message\ResponseInterface;
use WeeConnectPay\Api\ApiClient;
use WeeConnectPay\Api\ApiEndpoints;
use WeeConnectPay\WordPress\Plugin\includes\WeeConnectPayUtilities;

class FindOrCreateWoocommerceIntegrationRequest extends ApiClient {

	/**
	 * POST request
	 */
	public function POST(): ResponseInterface {
		return $this->client->post( ApiEndpoints::woocommerceIntegration(), self::setRequestBody() );
	}

	/**
	 * Set the request body
	 * @return array
	 */
	private static function setRequestBody(): array {
		global $wpdb, $wp_version;

		$domain                   = sanitize_url( $_SERVER['SERVER_NAME']);
		$woocommerce_settings_url = WeeConnectPayUtilities::getSettingsURL();

		return [
			'domain'             => $domain,
			'integrable'         => [
				'type'    => 'woocommerce',
				'version' => defined( 'WC_VERSION' ) ? WC_VERSION : null,
			],
			'integrable_backend' => [
				'type'         => 'wordpress',
				'home_url'     => home_url() ?? null,
				'site_url'     => site_url() ?? null,
				'settings_url' => $woocommerce_settings_url ?? null,
				'plugins_url'  => plugins_url() ?? null,
				'db_prefix'    => $wpdb->prefix ?? null,
				'version'      => $wp_version ?? null,
			],
		];
	}
}
