<?php

namespace WeeConnectPay\Integrations;

use WeeConnectPay\Dependency;
use WeeConnectPay\Exceptions\Codes\ExceptionCode;
use WeeConnectPay\Exceptions\InsufficientDependencyVersionException;
use WeeConnectPay\Exceptions\MissingDependencyException;
use WeeConnectPay\Exceptions\WeeConnectPayException;
use WeeConnectPay\StandardizedResponse;
use WeeConnectPay\WordPress\Plugin\includes\WeeConnectPayUtilities;

class DependencyChecker {

	/**
	 * External dependencies with their minimum version for the integration to run properly
	 *
	 * @since 1.3.9
	 * @updated 1.4.3
	 */
	public const DEPENDENCIES = [
		Dependency::PHP         => [ 7, 2, 0 ],
		Dependency::WORDPRESS   => [ 5, 4, 0 ],
		Dependency::WOOCOMMERCE => [ 3, 0, 4 ],
		Dependency::SSL         => [ 0, 0, 0 ], // Quick fix for dependencies without a version to respect type safety
		Dependency::PERMALINK   => [ 0, 0, 0 ], // Quick fix for dependencies without a version to respect type safety
	];

	/**
	 * Admin notice closure using the dependency exception message
	 *
	 * @param WeeConnectPayException $exception
	 *
	 * @return void
	 * @since 1.3.9
	 */
	public function notify( WeeConnectPayException $exception ) {
		add_action( 'admin_notices', function () use ( $exception ) {
			self::adminNoticeCallback( $exception );
		} );
	}

	/**
	 *
	 * @param WeeConnectPayException $exception
	 *
	 * @return void
	 * @since 1.3.9
	 */
	public static function adminNoticeCallback( WeeConnectPayException $exception ) {
		echo '<div class="notice notice-error">
			    <p><b>WeeConnectPay</b></p>
	            <p>' . esc_html( $exception->getMessage()) . '</p>
	          </div>';
	}

	/**
	 * @throws MissingDependencyException
	 * @throws InsufficientDependencyVersionException
	 */
	public function validateOnRun(): void {
		foreach ( self::DEPENDENCIES as $dependencyName => $minVerArray ) {
			$dependency = new Dependency( $dependencyName, $minVerArray );
			$dependency->validate();
		}
	}
}
