<?php

namespace WeeConnectPay\Integrations;

use WeeConnectPay\WordPress\Plugin\includes\WeeConnectPayUtilities;

class PaymentFields {


	/**
	 * Registers the needed scripts for the payment fields
	 * @since 2.0.0
	 */
	public function registerScripts(): PaymentFields {
        $cloverSdkUrl = ( WeeConnectPayUtilities::get_wp_env() === 'production' ) ? 'https://checkout.clover.com/sdk.js' : 'https://checkout.sandbox.dev.clover.com/sdk.js';
		wp_register_script( 'weeconnectpay-clover-sdk-js', $cloverSdkUrl, [], null, false );
		wp_register_script( 'weeconnectpay-payment-fields-js', WEECONNECTPAY_PLUGIN_URL . 'dist/js/payment-fields.js', array( 'weeconnectpay-clover-sdk-js' ), null, false );
		return $this;
	}

	/**
	 * Makes WordPress localized data available for the scripts needed in for the payment fields
	 * @since 2.0.0
	 */
	public function localizeData( array $script_data ): PaymentFields {
		wp_localize_script( 'weeconnectpay-payment-fields-js', 'WeeConnectPayPaymentFieldsData', $script_data );
		return $this;
	}

	/**
	 * Registers the CSS used by the payment fields
	 * @since 2.0.0
	 */
	public function registerStyles(): PaymentFields {
		wp_register_style( 'weeconnectpay-payment-fields-css', WEECONNECTPAY_PLUGIN_URL . 'site/css/weeconnect-public.css', array(), '2.0.0' );
//		wp_register_style( 'weeconnectpay-vendors-chunk-css', WEECONNECTPAY_PLUGIN_URL . 'dist/css/chunk-vendors.css', array(), '2.0.0' );
		return $this;
	}

	/**
	 * Enqueues the needed scripts and styles in the right order for the payment fields
	 * @since 2.0.0
	 */
	public function enqueueVueScriptsAndStyles(): PaymentFields {
		wp_enqueue_script( 'weeconnectpay-clover-sdk-js' );
		wp_enqueue_script( 'weeconnectpay-payment-fields-js' );
		wp_enqueue_style( 'weeconnectpay-payment-fields-css' );
		//wp_enqueue_style( 'weeconnectpay-vendors-chunk-css' );
		return $this;
	}


	/**
	 * Output the HTML elements required for payment processing
	 * @since 2.0.0
	 */
	public function outputElements(): PaymentFields {
		?>
		<div id="weeconnectpay-wc-fields">
            <div id="form-display-no-footer">
                <div class="form-row top-row full-width">
                    <div id="card-number" class="field card-number-field"></div>
                    <div class="input-errors" id="card-number-errors" role="alert"></div>
                </div>

                <div class="bottom-row-wrapper">
                    <div class="form-row bottom-row third-width">
                        <div id="card-date" class="field card-date-field"></div>
                        <div class="input-errors" id="card-date-errors" role="alert"></div>
                    </div>

                    <div class="form-row bottom-row third-width">
                        <div id="card-cvv" class="field card-cvv-field"></div>
                        <div class="input-errors" id="card-cvv-errors" role="alert"></div>
                    </div>

                    <div class="form-row bottom-row third-width">
                        <div id="card-postal-code" class="field card-postal-code-field"></div>
                        <div class="input-errors" id="card-postal-code-errors" role="alert"></div>
                    </div>
                </div>
                <div id="card-response" role="alert"></div>
                <div id="card-errors" role="alert"></div>
                <div class="clover-footer"></div>
            </div>
		</div>
        <input type="hidden" value="" name="token" id="wcp-token"/>
		<?php
        return $this;
	}

	/**
     *
	 * @param array $script_data
	 *
	 * @since 2.0.0
	 * @return void
	 */
	public function init( array $script_data) {
		$this->outputElements()
             ->registerScripts()
		     ->localizeData( $script_data )
		     ->registerStyles()
		     ->enqueueVueScriptsAndStyles();
	}

}
