<?php

/**
 * Fired when the plugin is uninstalled.
 *
 * When populating this file, consider the following flow
 * of control:
 *
 * - This method should be static
 * - Check if the $_REQUEST content actually is the plugin name
 * - Run an admin referrer check to make sure it goes through authentication
 * - Verify the output of $_GET makes sense
 * - Repeat with other user roles. Best directly by using the links/query string parameters.
 * - Repeat things for multisite. Once for a single site in the network, once sitewide.
 *
 * This file may be updated more in future version of the Boilerplate; however, this is the
 * general skeleton and outline for how the file should work.
 *
 * For more information, see the following discussion:
 * https://github.com/tommcfarlin/WordPress-Plugin-Boilerplate/pull/123#issuecomment-28541913
 *
 * @link       https://github.com/ParogDev
 * @updated    1.4.7
 * @since      1.0.0
 *
 * @package    WeeConnectPay
 */

// Exit if file is called directly -- Yes we still want to check that.
use WeeConnectPay\Integrations\IntegrationSettings;
use WeeConnectPay\Settings;

if (!defined('ABSPATH')) {
    exit;
}

// If uninstall not called from WordPress, then exit.
if (!defined('WP_UNINSTALL_PLUGIN')) {
    exit;
}

/**
 * @since 1.4.7
 * The class responsible for the plugin settings
 */
require_once plugin_dir_path( __FILE__  ) . 'includes/modules/WeeConnectPay/Integration/IntegrationSettings.php';


delete_option( IntegrationSettings::PLUGIN_OPTION_PREFIX . Settings::DB_KEY_SUFFIX_CLOVER_APP);
delete_option( IntegrationSettings::PLUGIN_OPTION_PREFIX . Settings::DB_KEY_SUFFIX_CLOVER_EMPLOYEE);
delete_option( IntegrationSettings::PLUGIN_OPTION_PREFIX . Settings::DB_KEY_SUFFIX_CLOVER_MERCHANT);
delete_option( IntegrationSettings::PLUGIN_OPTION_PREFIX . Settings::DB_KEY_SUFFIX_CLOVER_MERCHANT_APP_SUBSCRIPTION);
delete_option( IntegrationSettings::PLUGIN_OPTION_PREFIX . Settings::DB_KEY_SUFFIX_CLOVER_MERCHANT_PUBLIC_KEY);
delete_option( IntegrationSettings::PLUGIN_OPTION_PREFIX . Settings::DB_KEY_SUFFIX_JWT);
delete_option( IntegrationSettings::PLUGIN_OPTION_PREFIX . IntegrationSettings::DB_KEY_SUFFIX_INTEGRATION_UUID);
delete_option( IntegrationSettings::PLUGIN_OPTION_PREFIX . IntegrationSettings::DB_KEY_SUFFIX_PLATFORM_TYPE);

// WooCommerce Options
delete_option(IntegrationSettings::DB_KEY_WOOCOMMERCE_INTEGRATION);

// Should technically not be used anymore... but just in case
delete_option(IntegrationSettings::PLUGIN_OPTION_PREFIX . 'integration');
