<?php /** @noinspection PhpCSValidationInspection */

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://www.weeconnectpay.com/
 * @since             1.0.0
 * @package           WeeConnectPay
 *
 * @wordpress-plugin
 * Plugin Name:       WeeConnectPay
 * Plugin URI:        https://weeconnectpay.com/plugin?platform=wordpress
 * Description:       Integrate Clover Payments with your WooCommerce online store.
 * Version:           2.0.10
 * Author:            WeeConnectPay
 * Author URI:        https://weeconnectpay.com/
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       weeconnectpay
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if (!defined('WPINC')) {
    die;
}
const WEECONNECT_VERSION = '2.0.10';

define( 'WEECONNECTPAY_PLUGIN_URL', plugin_dir_url(__FILE__));
define( 'WEECONNECTPAY_PLUGIN_PATH', plugin_dir_path(__FILE__));

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/WeeConnectPayActivator.php
 */
function activate_weeconnectpay()
{
	require_once plugin_dir_path(__FILE__) . 'includes/WeeConnectPayActivator.php';
    WeeConnectPay\WordPress\Plugin\includes\WeeConnectPayActivator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/WeeConnectPayDeactivator.php
 */
function deactivate_weeconnectpay()
{
    require_once plugin_dir_path(__FILE__) . 'includes/WeeConnectPayDeactivator.php';
    WeeConnectPay\WordPress\Plugin\includes\WeeConnectPayDeactivator::deactivate();
}

register_activation_hook(__FILE__, 'activate_weeconnectpay');
register_deactivation_hook(__FILE__, 'deactivate_weeconnectpay');

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and site-facing site hooks.
 */
require plugin_dir_path(__FILE__) . 'includes/WeeConnectPay.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_weeconnectpay()
{
    $plugin = WeeConnectPay\WordPress\Plugin\includes\WeeConnectPay::get_instance();
    $plugin->run();
}

run_weeconnectpay();
